// C3Engine.h  --  External application interface to Creatures 3
// Written by Gavin Lambert for Mirality Systems
// Based on c3-engine.dylan and documentation created by Chris Double,
//   http://www.double.co.nz/creatures/developer/sharedmemory.htm

#ifndef __MSI_C3ENGINE_H
#define __MSI_C3ENGINE_H

// Note: this code is illustrative, rather than robust.  You'll probably want
// to do some better checking on the error return values.
/******  Typical code sample  ******\

	PCreaturesEngine engine;
	DWORD result;
	char game[24];
	
	CreaturesLastGame(game, 24);			// Find most recent game
	if((result = CreaturesConnect(game, &engine)) == CREATURES_SUCCESS) {
		// Connected successfully to game engine
		if((result = CreaturesExecute(engine, "outv 666")) == CREATURES_SUCCESS) {
			// Successfully executed
		} else {
			// Unsuccessful
		}
		// Retrieve result (either output or error message) and display
		DWORD size = CreaturesResult(engine, NULL, 0);
		LPBYTE buffer = new BYTE[size];
		CreaturesResult(engine, buffer, size);
		cout << buffer << endl;
		delete[] buffer;
	} else {
		// Failed to connect to game engine.  Probably not running.
	}
	// Remember, you need to call Disconnect EVEN IF CONNECT FAILED!
	CreaturesDisconnect(&engine);

\******                       ******/

// I wanted this to be as compatible as possible, so the functions use the
// standard C naming convention, and the __stdcall calling convention.
#ifdef __cplusplus
extern "C" {
#endif

// Define common game names
#define CREATURES_GAME_C3				"Creatures 3"
#define CREATURES_GAME_CA				"Creatures Village"
#define CREATURES_GAME_DS				"Docking Station"

// Define error codes returned by the functions
#define CREATURES_SUCCESS								0
#define CREATURES_ERROR									1
// The above two codes are returned by Connect, and also by the Send routines
// directly from the game engine.  The following error codes (Send-specific)
// are returned by the DLL, before your command even makes it to the engine.
// Therefore, in the above two cases the Result will be set to the engine's
// response (whether output, "OK", or an error message), and in the cases
// below there will be no Result.
#define CREATURES_TERMINATED						2
// CREATURES_TERMINATED is returned by the Send routines if it appears that
// the Creatures engine terminated either before or during the operation.
#define CREATURES_TOOLONG								3
// CREATURES_TOOLONG is returned by the Send routines if the commands/script
// you supply is too long to fit in the shared buffer.  The buffer is pretty
// large, so you should never actually get this.
#define CREATURES_UNRECOGNISED					4
// CREATURES_UNRECOGNISED is returned by the Send routines if they don't
// recognise the signature provided by the game engine.  When this occurs,
// it means that you're trying to access an engine that is using an
// incompatible communication protocol.  In this case all you can do is
// give up or try a different communications library :)

// Note: these functions use a PCreaturesEngine.  This is an opaque structure
// (meaning that your application doesn't know what is in it); this holds
// necessary state information for your connection to the Creatures engine.
struct CreaturesEngine;
typedef struct CreaturesEngine *PCreaturesEngine;

// Warning: the send routines (CreaturesRawSend, CreaturesExecute, and
// CreaturesScript) use blocking wait functions, so calling ANY of them will
// prevent your application from receiving any messages (or responding to any
// user interaction) until the function is completed.  This means that if you
// execute a particularly long section of CAOS code (using WAIT, for example)
// your program may become unresponsive.  Also note that they will also wait
// for all other programs to be finished talking to the engine, so your
// application may be adversely affected by someone else's application.
//
//
// CreaturesLastGame:
//   Returns the name of the most recently executed Creatures game.  If a game
//   engine is still running, this is probably it.  You need to pass in a
//   buffer allocated to at least 20 characters (preferably more, just in
//   case), and pass in that size as well.  If the buffer was too small, or
//   for any other reason the game name couldn't be determined, it will be set
//   to "".  Assuming no errors, you can pass this to CreaturesConnect.
void __stdcall CreaturesLastGame(char *game, DWORD size);
//
// CreaturesConnect:
//   Connects your application to a Creatures engine.  Sets the
//   PCreaturesEngine provided and returns CREATURES_SUCCESS if successful.
//   You MUST call CreaturesDisconnect (passing in the same PCreaturesEngine)
//   when you are finished, even if this doesn't return CREATURES_SUCCESS.
DWORD __stdcall CreaturesConnect(const char *game, PCreaturesEngine *pEngine);

// CreaturesDisconnect:
//   Disconnects your application from a Creatures engine.  You must call this
//   after previously calling CreaturesConnect, even if that call failed.
//   This call will destroy the PCreaturesEngine and set it to NULL.
void __stdcall CreaturesDisconnect(PCreaturesEngine *pEngine);

// CreaturesResult:
//   Obtains the last result generated by a send operation.  The contents of
//   this will vary depending on what you sent.  You may pass NULL as the
//   data pointer to determine the length of the result, so that you can then
//   allocate the appropriate buffer.  The return value is the length of the
//   result; if this is greater than dwMaxData, the pData buffer will not be
//   filled.
DWORD __stdcall CreaturesResult(PCreaturesEngine engine, LPBYTE pData,
																DWORD dwMaxData);

// CreaturesFreeResult:
//   Release internal memory of the last result.  This will automatically
//   occur the next time you send data to the engine, so you don't necessarily
//   need to call this.  However, it is a good idea to call this function if
//   there is going to be a significant delay before the next send.
void __stdcall CreaturesFreeResult(PCreaturesEngine engine);

// CreaturesRawSend:
//   Sends a raw command to the Creatures engine.  Not really recommended for
//   normal use, but you can if you want to :)
DWORD __stdcall CreaturesRawSend(PCreaturesEngine engine, const char *command);

// CreaturesExecute:
//   Executes some CAOS commands (for example, an agent injection).
DWORD __stdcall CreaturesExecute(PCreaturesEngine engine, const char *command);

// CreaturesScript:
//   Installs an event script in the current world's scriptorium.
DWORD __stdcall CreaturesScript(PCreaturesEngine engine, BYTE family,
										BYTE genus, WORD species, WORD scrnum, const char *script);

#ifdef __cplusplus
}
#endif

#endif
